package de.szut.lf8_starter.project;

import org.json.JSONObject;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import java.time.LocalDate;
import java.util.Arrays;

import static org.assertj.core.api.Assertions.assertThat;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.hamcrest.Matchers.is;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@AutoConfigureMockMvc(addFilters = false)
public class CreateProjectActionTest {
    @Autowired
    private MockMvc mockMvc;
    @Autowired
    private ProjectRepository projectRepository;

    @Test
    void test() throws Exception {
        String content = """
                {
                   "name": "name",
                   "leading_employee": 1,
                   "employees": [2, 3],
                   "contractor": 4,
                   "contractorName": "Peter File",
                   "comment": "goal of project",
                   "startDate": "01.01.2000",
                   "plannedEndDate": "01.01.2001"
                 }
                """;

        final var contentAsString = this.mockMvc.perform(
                post("/projects").content(content).contentType(MediaType.APPLICATION_JSON)
        )
                .andExpect(status().isCreated())
                .andExpect(jsonPath("name", is("name")))
                .andExpect(jsonPath("leadingEmployee", is(0)))
                .andExpect(jsonPath("employees", is(Arrays.asList(2, 3))))
                .andExpect(jsonPath("contractor", is(4)))
                .andExpect(jsonPath("contractorName", is("Peter File")))
                .andExpect(jsonPath("comment", is("goal of project")))
                .andExpect(jsonPath("startDate", is("01.01.2000")))
                .andExpect(jsonPath("plannedEndDate", is("01.01.2001")))
                .andReturn()
                .getResponse()
                .getContentAsString();

            final var id = Long.parseLong(new JSONObject(contentAsString).get("id").toString());

            final var project = this.projectRepository.findById(id);
            assertThat(project.get().getName()).isEqualTo("name");
            assertThat(project.get().getLeadingEmployee()).isEqualTo(0);
            assertThat(project.get().getContractor()).isEqualTo(4);
            assertThat(project.get().getContractorName()).isEqualTo("Peter File");
            assertThat(project.get().getComment()).isEqualTo("goal of project");
            assertThat(project.get().getStartDate()).isEqualTo(LocalDate.of(2000, 1, 1));
            assertThat(project.get().getPlannedEndDate()).isEqualTo(LocalDate.of(2001, 1, 1));
    }
}
